<?php
/*
 *  Copyright (C) 2006 CSIRO Australia
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  - Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  
 *  - Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  
 *  - Neither the name of the CSIRO nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ORGANISATION OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* These functions are wrappers for equivilent static methods in 
 * SimpleCMMLElement. They are provided to maintain consistency with the 
 * standard SimpleXMLElement functions.
 */ 
function simplecmml_load_anx_file( $filename ) {
  return SimpleCMMLElement::load_anx_file( $filename );
}
function simplecmml_load_cmml_file( $filename ) {
  return SimpleCMMLElement::load_cmml_file( $filename );
}
function simplecmml_load_string( $xml_str ) {
  return SimpleCMMLElement::load_string( $xml_str );
}
  
class SimpleCMMLElement extends SimpleXMLElement {

  const anxripper = 'anxrip';
  const max_size = 20000;

  public static function load_string( $xml_str ) {
    return simplexml_load_string( $xml_str, 'SimpleCMMLElement');
  }
  public static function load_cmml_file( $filename ) {
    return simplexml_load_file( $filename, 'SimpleCMMLElement');
  }
  public function load_anx_file( $filename ) {
  
    $xml_str = SimpleCMMLElement::anx_rip( $filename );
    return SimpleCMMLElement::load_string( $xml_str );  
  }

  protected static function anx_rip( $filename ) {
    
    if ( !file_exists( $filename ) ) {
      throw new Exception("File doesn't exist");
    }
    
    $handle = popen(SimpleCMMLElement::anxripper." $filename", "r");
    if( !$handle ) {
      throw new Exception("Unable to open pipe");
    }
    else {
      $xml_str = fread($handle, SimpleCMMLElement::max_size);
    }

    pclose( $handle );
    return $xml_str;
  }
  
  public function set_max_size( $new_max ) {
    $this->max_size = $new_max;
    return;
  }
  public function get_max_size() {
    return $this->max_size;
  } 
} 
