/*
 *  Copyright (C) 2006 CSIRO Australia
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  - Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  
 *  - Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  
 *  - Neither the name of the CSIRO nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ORGANISATION OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "phpanx_util.h"
#include "phpanx_general.h"

PHP_FUNCTION(anx_parse_time)
{
	ANNODEX *anx;
	int result, str_length;
	char* str;
	zval* anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", 
			&str, &str_length) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	result = anx_parse_time( str );

	RETURN_LONG(result);
}

PHP_FUNCTION(anx_last_error)
{
	ANNODEX *anx;
	int result;
	zval* anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	result = anx_last_error( anx );

	RETURN_LONG(result);
}

PHP_FUNCTION(anx_strerror)
{
	ANNODEX *anx;
	const char *error;
	zval* anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	error = anx_strerror( anx );

	RETURN_STRING( error, DUPLICATE);
}

PHP_FUNCTION(anx_open)
{
	char *filename;
	int filename_length, resource_id;
	long mode;
	ANNODEX* anx;
	AnxClip my_clip;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl", 
			&filename, &filename_length, &mode) == FAILURE) {
		RETURN_NULL();
	}
	
	/* Call Annodex function */
	if( mode == ANX_WRITE ) {
		anx_init_importers("*/*");
	}
	anx = anx_open(filename, mode);

	if( anx == NULL ) {
		RETURN_NULL();
	}

	resource_id = anx_to_resource_id( anx );
	RETURN_RESOURCE( resource_id );
	

}

PHP_FUNCTION(anx_new)
{
	int mode;
	ANNODEX* anx;
	zval* anx_resource;
	int resource_id;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "l", 
			&mode) == FAILURE) {
		RETURN_NULL();
	}

	/* Call Annodex function */
	anx = anx_new( mode );

	/* Return NULL if call to anx_new() fails */
	if( anx == NULL ) {
		RETURN_NULL();
	}  
	
	resource_id = anx_to_resource_id( anx );
	RETURN_RESOURCE( resource_id );
}

PHP_FUNCTION(anx_flush)
{
	zval* anx_resource;
	ANNODEX *anx = NULL;
	int result;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
			LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	result = anx_flush( anx );

	if( result == 0 ) {
		RETURN_TRUE;
	} else {
		RETURN_FALSE;
	}
	
}

PHP_FUNCTION(anx_close)
{
	zval *anx_resource, *returned_anx_resource;
	ANNODEX *anx = NULL;
	int returned_resource_id;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
			LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	anx = anx_close( anx );

	if( anx != NULL ) {
		RETURN_FALSE;
		//returned_resource_id = anx_to_resource_id( NULL );
		//RETURN_RESOURCE( returned_resource_id );
	} else {
		RETURN_TRUE;
	}
	
}

PHP_FUNCTION(anx_destroy)
{
	zval* anx_resource;
	ANNODEX *anx = NULL;
	int result;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
			LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	result = anx_destroy( anx );

	if( result == 0 ) {
		RETURN_TRUE;
	} else {
		RETURN_FALSE;
	}
	
}

PHP_FUNCTION(anx_ready)
{
	ANNODEX *anx;
	int is_ready;
	zval *anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	is_ready = anx_ready( anx );

	RETURN_BOOL( is_ready );
}

PHP_FUNCTION(anx_eos)
{
	ANNODEX *anx;
	int result;
	zval *anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	result = anx_eos( anx );

	RETURN_BOOL( result );
}

PHP_FUNCTION(anx_set_head)
{
	ANNODEX *anx;
	AnxHead *head;
	zval *anx_resource, *head_object;
	int resource_id;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rz", 
			&anx_resource, &head_object ) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);
	object_init( head_object );
	head = object_to_head( head_object );
	
	/* Call Annodex function */
	anx = anx_set_head( anx, head );

	resource_id = anx_to_resource_id( anx );
	RETURN_RESOURCE( resource_id );
}
PHP_FUNCTION(anx_get_presentation_time)
{
	ANNODEX *anx;
	double presentation_time;
	zval *anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	presentation_time = anx_get_presentation_time( anx );

	RETURN_DOUBLE( presentation_time );
}


PHP_FUNCTION(anx_get_basetime)
{
	ANNODEX *anx;
	double basetime;
	zval *anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	basetime = anx_get_basetime( anx );

	RETURN_DOUBLE( basetime );
}

PHP_FUNCTION(anx_get_bitrate)
{
	ANNODEX *anx;
	double bitrate;
	zval *anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	bitrate = anx_get_bitrate( anx );

	RETURN_DOUBLE( bitrate );
}

PHP_FUNCTION(anx_get_duration)
{
	ANNODEX *anx;
	double duration;
	zval *anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	duration = anx_get_duration( anx );

	RETURN_DOUBLE( duration );
}

PHP_FUNCTION(anx_tell_time)
{
	ANNODEX *anx;
	double offset;
	zval *anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	offset = anx_tell_time( anx );

	RETURN_DOUBLE( offset );
}

PHP_FUNCTION(anx_seek_time)
{
	ANNODEX *anx;
	int whence;
	double seconds, result;
	zval *anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rdl", 
			&anx_resource, &seconds, &whence) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	result = anx_seek_time( anx, seconds, whence );

	RETURN_DOUBLE( result );
}
