/*
 *  Copyright (C) 2006 CSIRO Australia
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  - Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  
 *  - Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  
 *  - Neither the name of the CSIRO nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE ORGANISATION OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "phpanx_util.h"
#include "phpanx_write.h"

#define PERSISTENT	1
#define NON_PERSISTENT	0

PHP_FUNCTION(anx_init_importers)
{

	char* content_type_pattern;
	int string_length;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", 
			&content_type_pattern, &string_length) == FAILURE) {
		RETURN_NULL();
	}

	/* Call Annodex function */
	anx_init_importers( content_type_pattern );
	
	RETURN_TRUE;
}

PHP_FUNCTION(anx_write_get_next_page_size)
{

	zval *anx_resource;
	long next_page_size;
	ANNODEX *anx;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "r", 
			&anx_resource ) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	
	/* Call Annodex function */
	anx_write_get_next_page_size( anx );
	
	RETURN_LONG( next_page_size );
}

PHP_FUNCTION(anx_write_import)
{

	ANNODEX *anx;
	char *filename, *id, *content_type;
	int filename_length, id_length, content_type_length;
	double seek_offset, seek_end;
	int flags, result;
	zval *anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rsssddd", 
			&anx_resource, &filename, &filename_length, &id, 
			&id_length, &content_type, &content_type_length, 
			&seek_offset, &seek_end, &flags ) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Convert empty strings to NULLs */
	id = empty_to_null( id );
	content_type = empty_to_null( content_type );

	/* Call Annodex function */
	result = anx_write_import( anx, filename, id, content_type, seek_offset,
			 seek_end, flags);

	RETURN_LONG( result );
}



PHP_FUNCTION(anx_insert_clip)
{

	ANNODEX *anx;
	AnxClip *clip=NULL;
	double at_time;
	int result;
	zval *anx_resource, *clip_object;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rdz", 
			&anx_resource, &at_time, &clip_object ) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);
	object_init( clip_object );
	//clip = object_to_clip( clip_object );

	/* Call Annodex function */
	result = anx_insert_clip( anx, at_time, clip );
	
	RETURN_LONG( result );
}

PHP_FUNCTION(anx_insert_cmml)
{

	ANNODEX *anx;
	long n;
	double at_time;
	const char *cmml;
	int cmml_length, result;
	zval *anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rdsl", 
			&anx_resource, &at_time, &cmml, &cmml_length, &n ) 
			== FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	result = anx_insert_cmml( anx, at_time, cmml, n );

	if( result == 0 ) {
		RETURN_TRUE;
	} else {
		RETURN_FALSE;
	}
}

PHP_FUNCTION(anx_write)
{
	long n;
	ANNODEX *anx;
	int result;
	zval *anx_resource;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", 
			&anx_resource, &n) == FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	/* Call Annodex function */
	result = anx_write( anx, n );

	RETURN_LONG( result );
}

PHP_FUNCTION(anx_write_output)
{
	long n;
	ANNODEX *anx;
	int bytes_written;
	zval *anx_resource;
	unsigned char* buffer;

	/* Process arguments */
	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl", 
			&anx_resource, &n) ==FAILURE) {
		RETURN_NULL();
	}
	ZEND_FETCH_RESOURCE( anx, ANNODEX*, &anx_resource, -1, 
		LE_ANX_HANDLE_NAME, le_anx_handle);

	buffer = pemalloc( n * sizeof(char) , NON_PERSISTENT );

	if( buffer == NULL ) {
		RETURN_FALSE;
	}
	
	/* Call Annodex function */
	if ( (bytes_written = anx_write_output( anx, buffer, n )) > 0){

		RETVAL_STRINGL(buffer, bytes_written, DUPLICATE );
		pefree( buffer, NON_PERSISTENT );
		return;
	} else {
		pefree( buffer, NON_PERSISTENT );
		RETURN_FALSE;
	}
}

